import com.millistream.www.Mdf;
import com.millistream.www.Message;
import com.millistream.www.MessageField;
import com.millistream.www.MessageHeader;
import com.millistream.www.fields.F;
import com.millistream.www.fields.M;
import com.millistream.www.fields.RC;
import com.millistream.www.fields.RT;

import java.util.ArrayList;
import java.util.Properties;

public class Example {
	public static void main(String[] args) {
		String server = "";
		String username = "";
		String password = "";

		Mdf mdf = new Mdf();
		Message message = new Message();

		for (int i = 0; i < args.length; i++) {
			if (args[i].equals ("--server")) {
				server = args[i + 1];
			} else if (args[i].equals ("--username")) {
				username = args[i + 1];
			} else if (args[i].equals ("--password")) {
				password = args[i + 1];
			} else {
				System.out.print ("unknown argument: " + args[i] + "\n");
				continue;
			}

			i++;
		}

		if (server.isEmpty())
			server = "sandbox.millistream.com:9100";

		if (username.isEmpty() || password.isEmpty()) {
			System.out.println ("usage: example --username <username> --password <password> [--server <server>]");
			System.exit (1);
		}

		if (!mdf.connect (server)) {
			System.out.println ("unable to connect with server: " + server);
			System.exit (1);
		}

		/* construct a simple application / user-agent that we send on logon
		 * to the server. This is completely optional but it makes it easier
		 * for Millistream Tech Support to distinguish between all the different
		 * connections. */
		Properties prop = System.getProperties();
		String application = new String("libmdf Java JNI Example");

		application += " (";
		application += prop.getProperty("java.vendor");
		application += prop.getProperty("java.version");
		application += ")";

		/* logon to the server we just connected with */
		if (!send_logon (mdf, message, application, username, password))
			System.exit (1);

		/* here we send a request for three insrefs:
		 * 772 - 'Ericsson B' which is a Share instrument
		 * 26021 - 'Bats Europe CXE' which is a Market Place instrument
		 * 907169 - 'XCSE Other Equities' which is a Tick Table instrument
		 *
		 * Depending upon your account you might not have access to these
		 * particular instruments! */
		send_request (mdf, message, "772 26021 907169");

		while (true) {
			/* we loop until we get disconnected or have a network error */
			int ret = mdf.consume (1);

			if (ret == -1) /* disconnected or network error */
				break;

			if (ret == 1) { /* we have data! */
				for (MessageHeader mh : mdf.getMessagesAndFields()) {
					System.out.println(mh);

					for (MessageField mf : mh.getFields()) {
						System.out.println(mf);

						/* some fields are in tabular format */
						if (mf.getTag() == F.CONSTITUENTS || mf.getTag() == F.TICKSIZES || mf.getTag() == F.BROKERS || mf.getTag() == F.UNDERLYINGID || mf.getTag() == F.COMBOLEGS || mf.getTag() == F.NEWSCODINGTAGS) {
							System.out.println("\t\tThis data is in tabular format, let's break up the data in rows and columns:");

							int i = 0;
							for (String[] row : mf.getTabularValue()) {
								System.out.print ("\t\tRow #" + i++ + ":");
								for (String column : row) {
									System.out.print ("\t" + column);
								}

								System.out.println ();
							}
						}
					}

					System.out.println();
				}
			}
		}

		send_logoff (mdf, message);
	}

	/* sends a logon request to the server and awaits a reply from it,
	 * we wait a maximum of 10 seconds for the server reply before we timeout */
	public static boolean send_logon(Mdf mdf, Message message, String application, String username, String password) {
		message.add (0, M.LOGON);
		message.add_string (F.USERNAME, username);
		message.add_string (F.PASSWORD, password);
		message.add_string (F.S1, application); /* optional application/user-agent */
		mdf.send(message);
		message.reset();

		long now = System.currentTimeMillis();

		while (System.currentTimeMillis() - now < 10 * 1000) {
			int ret = mdf.consume (1);

			if (ret == 1) {
				MessageHeader mh;

				while ((mh = mdf.get_next_message()) != null) {
					if (mh.getMref() == M.LOGONGREETING) {
						return true;
					} else if (mh.getMref() == M.LOGOFF) {
						System.out.println ("server sent logoff!"); 
						return false;
					}
				}
			} else if (ret == -1) {
				System.out.println ("error on read from server");
				return false;
			}
		}

		System.out.println ("send_logon() timed out!");
		return false;
	}

	/* send a logoff message to the server and awaits
	 * a reply from it, this way we can ensure that it has
	 * consumed all the data we have sent to it (only really important
	 * if we are a feed producer for Millistream, but still :-) */
	public static void send_logoff(Mdf mdf, Message message) {
		message.add (0, M.LOGOFF);
		mdf.send(message);
		message.reset();

		long now = System.currentTimeMillis();

		while (System.currentTimeMillis() - now < 10 * 1000) {
			int ret = mdf.consume (1);

			if (ret == 1) {
				MessageHeader mh;

				while ((mh = mdf.get_next_message()) != null) {
					if (mh.getMref() == M.LOGOFF)
						return;
				}
			} else if (ret == -1) {
				return;
			}
		}
	}

	/* send a request to the server for Basic Data and Quote for the supplied
	 * list of instruments. If the list is NULL then we issue a wildcard request
	 * i.e make the server send data for all available instruments 
	 *
	 * Please note that we use a condensed request, i.e we issue the basic data
	 * and the quote request in a single request. When doing that we have to use
	 * the add_list() method and we also have to put a " " between
	 * the request classes since the string has to be space separated */
	public static void send_request(Mdf mdf, Message message, String request) {
		message.add (0, M.REQUEST);
		message.add_list (F.REQUESTCLASS, RC.BASICDATA + " " + RC.QUOTE);
		message.add_numeric (F.REQUESTTYPE, RT.FULL);

		if (request != null)
			message.add_list (F.INSREFLIST, request);
		else
			message.add_string (F.INSREFLIST, "*");

		mdf.send(message);
		message.reset();
	}
}

