#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <mdf/mdf.h>
#include <mdf/mdf_fields.h>
#include "common.h"

int send_logon (mdf_t mdf, mdf_message_t message, char *application, char *username, char *password)
{
	time_t now;
	mdf_message_add (message, 0, MDF_M_LOGON);
	mdf_message_add_string (message, MDF_F_USERNAME, username);
	mdf_message_add_string (message, MDF_F_PASSWORD, password);
	mdf_message_add_string (message, MDF_F_S1, application); /* optional application/user-agent string */
	mdf_message_send (mdf, message);
	mdf_message_reset (message);

	now = time (NULL);

	while (time (NULL) - now < 10) {
		int ret = mdf_consume (mdf, 1);

		if (ret == 1) {
			uint64_t insref;
			uint16_t mref;

			while (mdf_get_next_message2 (mdf, &mref, &insref)) {
				if (mref == MDF_M_LOGONGREETING)
					return 0;

				if (mref == MDF_M_LOGOFF) {
					printf ("server sent logoff!\n"); 
					return -1;
				}
			}
		}

		if (ret == -1) {
			printf ("error on read from server!\n"); 
			return -1;
		}
	}

	printf ("send_logon() timed out!\n"); 
	return -1;
}

void send_logoff (mdf_t mdf, mdf_message_t message)
{
	/* we send a logoff message to the server and awaits
	 * a reply from it, this way we can ensure that it has
	 * consumed all the data we have sent to it (only really important
	 * if we are a feed producer for Millistream, but still :-) */
	time_t now;
	mdf_message_reset (message);
	mdf_message_add (message, 0, MDF_M_LOGOFF);

	if (mdf_message_send (mdf, message) != 1)
		return;

	now = time (NULL);

	while (time (NULL) - now < 10) {
		int ret = mdf_consume (mdf, 1);

		if (ret == 1) {
			uint64_t insref;
			uint16_t mref;

			while (mdf_get_next_message2 (mdf, &mref, &insref)) {
				if (mref == MDF_M_LOGOFF)
					return;
			}
		}

		if (ret == -1)
			break;
	}
}

void print_header (const uint16_t mref, const uint64_t mclass, const uint64_t insref)
{
	printf ("mref: ");

	switch (mref) {
		case MDF_M_MESSAGESREFERENCE : printf ("MDF_M_MESSAGESREFERENCE"); break;
		case MDF_M_LOGON             : printf ("MDF_M_LOGON"); break;
		case MDF_M_LOGOFF            : printf ("MDF_M_LOGOFF"); break;
		case MDF_M_LOGONGREETING     : printf ("MDF_M_LOGONGREETING"); break;
		case MDF_M_NEWSHEADLINE      : printf ("MDF_M_NEWSHEADLINE"); break;
		case MDF_M_QUOTE             : printf ("MDF_M_QUOTE"); break;
		case MDF_M_TRADE             : printf ("MDF_M_TRADE"); break;
		case MDF_M_BIDLEVELINSERT    : printf ("MDF_M_BIDLEVELINSERT"); break;
		case MDF_M_ASKLEVELINSERT    : printf ("MDF_M_ASKLEVELINSERT"); break;
		case MDF_M_BIDLEVELDELETE    : printf ("MDF_M_BIDLEVELDELETE"); break;
		case MDF_M_ASKLEVELDELETE    : printf ("MDF_M_ASKLEVELDELETE"); break;
		case MDF_M_BIDLEVELUPDATE    : printf ("MDF_M_BIDLEVELUPDATE"); break;
		case MDF_M_ASKLEVELUPDATE    : printf ("MDF_M_ASKLEVELUPDATE"); break;
		case MDF_M_INSTRUMENTRESET   : printf ("MDF_M_INSTRUMENTRESET"); break;
		case MDF_M_ORDERBOOKFLUSH    : printf ("MDF_M_ORDERBOOKFLUSH"); break;
		case MDF_M_BASICDATA         : printf ("MDF_M_BASICDATA"); break;
		case MDF_M_PRICEHISTORY      : printf ("MDF_M_PRICEHISTORY"); break;
		case MDF_M_INSTRUMENTDELETE  : printf ("MDF_M_INSTRUMENTDELETE"); break;
		case MDF_M_FIELDSREFERENCE   : printf ("MDF_M_FIELDSREFERENCE"); break;
		case MDF_M_REQUEST           : printf ("MDF_M_REQUEST"); break;
		case MDF_M_REQUESTFINISHED   : printf ("MDF_M_REQUESTFINISHED"); break;
		case MDF_M_INSREF            : printf ("MDF_M_INSREF"); break;
		case MDF_M_NEWSCONTENT       : printf ("MDF_M_NEWSCONTENT"); break;
		case MDF_M_TRADESTATE        : printf ("MDF_M_TRADESTATE"); break;
		case MDF_M_PERFORMANCE       : printf ("MDF_M_PERFORMANCE"); break;
		case MDF_M_KEYRATIOS         : printf ("MDF_M_KEYRATIOS"); break;
		case MDF_M_ESTIMATES         : printf ("MDF_M_ESTIMATES"); break;
		case MDF_M_ESTIMATESHISTORY  : printf ("MDF_M_ESTIMATESHISTORY"); break;
		case MDF_M_NETORDERIMBALANCE : printf ("MDF_M_NETORDERIMBALANCE"); break;
		case MDF_M_L10N              : printf ("MDF_M_L10N"); break;
		case MDF_M_CI                : printf ("MDF_M_CI"); break;
		case MDF_M_CIHISTORY         : printf ("MDF_M_CIHISTORY"); break;
		case MDF_M_PRIIP             : printf ("MDF_M_PRIIP"); break;
		case MDF_M_MIFID             : printf ("MDF_M_MIFID"); break;
		case MDF_M_MIFIDHISTORY      : printf ("MDF_M_MIFIDHISTORY"); break;
		case MDF_M_MAPPINGS          : printf ("MDF_M_MAPPINGS"); break;
		case MDF_M_MBOADD            : printf ("MDF_M_MBOADD"); break;
		case MDF_M_MBOUPDATE         : printf ("MDF_M_MBOUPDATE"); break;
		case MDF_M_MBODELETE         : printf ("MDF_M_MBODELETE"); break;
		case MDF_M_GREEKS            : printf ("MDF_M_GREEKS"); break;
		case MDF_M_QUOTEBBO          : printf ("MDF_M_QUOTEBBO"); break;
		case MDF_M_QUOTEEX           : printf ("MDF_M_QUOTEEX"); break;
		default                      : printf ("unknown");
	}

	printf ("[%hu], mclass:", mref);

	if (mclass == MDF_MC_UNDEF)                                          printf (" MDF_MC_UNDEF");
	if ((mclass & MDF_MC_NEWSHEADLINE)      == MDF_MC_NEWSHEADLINE)      printf (" MDF_MC_NEWSHEADLINE");
	if ((mclass & MDF_MC_QUOTE)             == MDF_MC_QUOTE)             printf (" MDF_MC_QUOTE");
	if ((mclass & MDF_MC_TRADE)             == MDF_MC_TRADE)             printf (" MDF_MC_TRADE");
	if ((mclass & MDF_MC_ORDER)             == MDF_MC_ORDER)             printf (" MDF_MC_ORDER");
	if ((mclass & MDF_MC_BASICDATA)         == MDF_MC_BASICDATA)         printf (" MDF_MC_BASICDATA");
	if ((mclass & MDF_MC_PRICEHISTORY)      == MDF_MC_PRICEHISTORY)      printf (" MDF_MC_PRICEHISTORY");
	if ((mclass & MDF_MC_NEWSCONTENT)       == MDF_MC_NEWSCONTENT)       printf (" MDF_MC_NEWSCONTENT");
	if ((mclass & MDF_MC_TRADESTATE)        == MDF_MC_TRADESTATE)        printf (" MDF_MC_TRADESTATE");
	if ((mclass & MDF_MC_FUNDAMENTALS)      == MDF_MC_FUNDAMENTALS)      printf (" MDF_MC_FUNDAMENTALS");
	if ((mclass & MDF_MC_PERFORMANCE)       == MDF_MC_PERFORMANCE)       printf (" MDF_MC_PERFORMANCE");
	if ((mclass & MDF_MC_KEYRATIOS)         == MDF_MC_KEYRATIOS)         printf (" MDF_MC_KEYRATIOS");
	if ((mclass & MDF_MC_ESTIMATES)         == MDF_MC_ESTIMATES)         printf (" MDF_MC_ESTIMATES");
	if ((mclass & MDF_MC_ESTIMATESHISTORY)  == MDF_MC_ESTIMATESHISTORY)  printf (" MDF_MC_ESTIMATESHSITORY");
	if ((mclass & MDF_MC_NETORDERIMBALANCE) == MDF_MC_NETORDERIMBALANCE) printf (" MDF_MC_NETORDERIMBALANCE");
	if ((mclass & MDF_MC_L10N)              == MDF_MC_L10N)              printf (" MDF_MC_L10N");
	if ((mclass & MDF_MC_CI)                == MDF_MC_CI)                printf (" MDF_MC_CI");
	if ((mclass & MDF_MC_CIHISTORY)         == MDF_MC_CIHISTORY)         printf (" MDF_MC_CIHISTORY");
	if ((mclass & MDF_MC_PRIIP)             == MDF_MC_PRIIP)             printf (" MDF_MC_PRIIP");
	if ((mclass & MDF_MC_MIFID)             == MDF_MC_MIFID)             printf (" MDF_MC_MIFID");
	if ((mclass & MDF_MC_MAPPINGS)          == MDF_MC_MAPPINGS)          printf (" MDF_MC_MAPPINGS");
	if ((mclass & MDF_MC_MBO)               == MDF_MC_MBO)               printf (" MDF_MC_MBO");
	if ((mclass & MDF_MC_GREEKS)            == MDF_MC_GREEKS)            printf (" MDF_MC_GREEKS");
	if ((mclass & MDF_MC_QUOTEBBO)          == MDF_MC_QUOTEBBO)          printf (" MDF_MC_QUOTEBBO");
	if ((mclass & MDF_MC_QUOTEEX)           == MDF_MC_QUOTEEX)           printf (" MDF_MC_QUOTEEX");

	printf ("[%lu], insref: %lu\n", mclass, insref);
}

/* generated by: awk '{ print $2 };' src/mdf_fields.h | grep MDF_F_ | awk '{ print "\t\tcase " $1 " : printf (\"" $1 "\"); break;" };' */

void print_field (const uint32_t tag, char * const value)
{
	printf ("  %d.", tag);

	switch (tag) {
		case MDF_F_LANGUAGE : printf ("MDF_F_LANGUAGE"); break;
		case MDF_F_HEADLINE : printf ("MDF_F_HEADLINE"); break;
		case MDF_F_TEXTBODY : printf ("MDF_F_TEXTBODY"); break;
		case MDF_F_DATE : printf ("MDF_F_DATE"); break;
		case MDF_F_TIME : printf ("MDF_F_TIME"); break;
		case MDF_F_BIDPRICE : printf ("MDF_F_BIDPRICE"); break;
		case MDF_F_ASKPRICE : printf ("MDF_F_ASKPRICE"); break;
		case MDF_F_LASTPRICE : printf ("MDF_F_LASTPRICE"); break;
		case MDF_F_DAYHIGHPRICE : printf ("MDF_F_DAYHIGHPRICE"); break;
		case MDF_F_DAYLOWPRICE : printf ("MDF_F_DAYLOWPRICE"); break;
		case MDF_F_QUANTITY : printf ("MDF_F_QUANTITY"); break;
		case MDF_F_TURNOVER : printf ("MDF_F_TURNOVER"); break;
		case MDF_F_TRADEPRICE : printf ("MDF_F_TRADEPRICE"); break;
		case MDF_F_TRADEQUANTITY : printf ("MDF_F_TRADEQUANTITY"); break;
		case MDF_F_TRADEREFERENCE : printf ("MDF_F_TRADEREFERENCE"); break;
		case MDF_F_TRADECODE : printf ("MDF_F_TRADECODE"); break;
		case MDF_F_ORDERLEVEL : printf ("MDF_F_ORDERLEVEL"); break;
		case MDF_F_NUMBIDORDERS : printf ("MDF_F_NUMBIDORDERS"); break;
		case MDF_F_NUMASKORDERS : printf ("MDF_F_NUMASKORDERS"); break;
		case MDF_F_BIDQUANTITY : printf ("MDF_F_BIDQUANTITY"); break;
		case MDF_F_ASKQUANTITY : printf ("MDF_F_ASKQUANTITY"); break;
		case MDF_F_SYMBOL : printf ("MDF_F_SYMBOL"); break;
		case MDF_F_NAME : printf ("MDF_F_NAME"); break;
		case MDF_F_ISIN : printf ("MDF_F_ISIN"); break;
		case MDF_F_BOARDLOT : printf ("MDF_F_BOARDLOT"); break;
		case MDF_F_INSTRUMENTTYPE : printf ("MDF_F_INSTRUMENTTYPE"); break;
		case MDF_F_INSTRUMENTSUBTYPE : printf ("MDF_F_INSTRUMENTSUBTYPE"); break;
		case MDF_F_DERIVATIVEINDICATOR : printf ("MDF_F_DERIVATIVEINDICATOR"); break;
		case MDF_F_EXERCISETYPE : printf ("MDF_F_EXERCISETYPE"); break;
		case MDF_F_ISSUECURRENCY : printf ("MDF_F_ISSUECURRENCY"); break;
		case MDF_F_TRADECURRENCY : printf ("MDF_F_TRADECURRENCY"); break;
		case MDF_F_BASECURRENCY : printf ("MDF_F_BASECURRENCY"); break;
		case MDF_F_QUOTECURRENCY : printf ("MDF_F_QUOTECURRENCY"); break;
		case MDF_F_ISSUEDATE : printf ("MDF_F_ISSUEDATE"); break;
		case MDF_F_STRIKEDATE : printf ("MDF_F_STRIKEDATE"); break;
		case MDF_F_STRIKEPRICE : printf ("MDF_F_STRIKEPRICE"); break;
		case MDF_F_TRADETIME : printf ("MDF_F_TRADETIME"); break;
		case MDF_F_NUMTRADES : printf ("MDF_F_NUMTRADES"); break;
		case MDF_F_EXECUTEDSIDE : printf ("MDF_F_EXECUTEDSIDE"); break;
		case MDF_F_OPENPRICE : printf ("MDF_F_OPENPRICE"); break;
		case MDF_F_CLOSEPRICE : printf ("MDF_F_CLOSEPRICE"); break;
		case MDF_F_CLOSEBIDPRICE : printf ("MDF_F_CLOSEBIDPRICE"); break;
		case MDF_F_CLOSEASKPRICE : printf ("MDF_F_CLOSEASKPRICE"); break;
		case MDF_F_CLOSEDAYHIGHPRICE : printf ("MDF_F_CLOSEDAYHIGHPRICE"); break;
		case MDF_F_CLOSEDAYLOWPRICE : printf ("MDF_F_CLOSEDAYLOWPRICE"); break;
		case MDF_F_CLOSEQUANTITY : printf ("MDF_F_CLOSEQUANTITY"); break;
		case MDF_F_CLOSETURNOVER : printf ("MDF_F_CLOSETURNOVER"); break;
		case MDF_F_CLOSENUMTRADES : printf ("MDF_F_CLOSENUMTRADES"); break;
		case MDF_F_NEWSID : printf ("MDF_F_NEWSID"); break;
		case MDF_F_REQUESTID : printf ("MDF_F_REQUESTID"); break;
		case MDF_F_REQUESTSTATUS : printf ("MDF_F_REQUESTSTATUS"); break;
		case MDF_F_REQUESTTYPE : printf ("MDF_F_REQUESTTYPE"); break;
		case MDF_F_REQUESTCLASS : printf ("MDF_F_REQUESTCLASS"); break;
		case MDF_F_INSREFLIST : printf ("MDF_F_INSREFLIST"); break;
		case MDF_F_MARKETPLACE : printf ("MDF_F_MARKETPLACE"); break;
		case MDF_F_LIST : printf ("MDF_F_LIST"); break;
		case MDF_F_INTERNALQUANTITY : printf ("MDF_F_INTERNALQUANTITY"); break;
		case MDF_F_INTERNALTURNOVER : printf ("MDF_F_INTERNALTURNOVER"); break;
		case MDF_F_CLOSEINTERNALQUANTITY : printf ("MDF_F_CLOSEINTERNALQUANTITY"); break;
		case MDF_F_CLOSEINTERNALTURNOVER : printf ("MDF_F_CLOSEINTERNALTURNOVER"); break;
		case MDF_F_TRADEBUYER : printf ("MDF_F_TRADEBUYER"); break;
		case MDF_F_TRADESELLER : printf ("MDF_F_TRADESELLER"); break;
		case MDF_F_BIDCOUNTERPART : printf ("MDF_F_BIDCOUNTERPART"); break;
		case MDF_F_ASKCOUNTERPART : printf ("MDF_F_ASKCOUNTERPART"); break;
		case MDF_F_COMPANY : printf ("MDF_F_COMPANY"); break;
		case MDF_F_FUNDPPMCODE : printf ("MDF_F_FUNDPPMCODE"); break;
		case MDF_F_UNDERLYINGID : printf ("MDF_F_UNDERLYINGID"); break;
		case MDF_F_USERNAME : printf ("MDF_F_USERNAME"); break;
		case MDF_F_PASSWORD : printf ("MDF_F_PASSWORD"); break;
		case MDF_F_EXTRACREDENTIAL : printf ("MDF_F_EXTRACREDENTIAL"); break;
		case MDF_F_LOGOFFREASON : printf ("MDF_F_LOGOFFREASON"); break;
		case MDF_F_TRADETYPE : printf ("MDF_F_TRADETYPE"); break;
		case MDF_F_TRADECANCELTIME : printf ("MDF_F_TRADECANCELTIME"); break;
		case MDF_F_NEWSBLOCKNUMBER : printf ("MDF_F_NEWSBLOCKNUMBER"); break;
		case MDF_F_BIDYIELD : printf ("MDF_F_BIDYIELD"); break;
		case MDF_F_ASKYIELD : printf ("MDF_F_ASKYIELD"); break;
		case MDF_F_LASTYIELD : printf ("MDF_F_LASTYIELD"); break;
		case MDF_F_OPENYIELD : printf ("MDF_F_OPENYIELD"); break;
		case MDF_F_DAYHIGHYIELD : printf ("MDF_F_DAYHIGHYIELD"); break;
		case MDF_F_DAYLOWYIELD : printf ("MDF_F_DAYLOWYIELD"); break;
		case MDF_F_CLOSEBIDYIELD : printf ("MDF_F_CLOSEBIDYIELD"); break;
		case MDF_F_CLOSEASKYIELD : printf ("MDF_F_CLOSEASKYIELD"); break;
		case MDF_F_CLOSEYIELD : printf ("MDF_F_CLOSEYIELD"); break;
		case MDF_F_CLOSEDAYHIGHYIELD : printf ("MDF_F_CLOSEDAYHIGHYIELD"); break;
		case MDF_F_CLOSEDAYLOWYIELD : printf ("MDF_F_CLOSEDAYLOWYIELD"); break;
		case MDF_F_NEWSCODINGCOMPANY : printf ("MDF_F_NEWSCODINGCOMPANY"); break;
		case MDF_F_NEWSCODINGTYPE : printf ("MDF_F_NEWSCODINGTYPE"); break;
		case MDF_F_NEWSCODINGSUBJECT : printf ("MDF_F_NEWSCODINGSUBJECT"); break;
		case MDF_F_NEWSCODINGCOUNTRY : printf ("MDF_F_NEWSCODINGCOUNTRY"); break;
		case MDF_F_NEWSCODINGORIGINAL : printf ("MDF_F_NEWSCODINGORIGINAL"); break;
		case MDF_F_FUNDCOMPANY : printf ("MDF_F_FUNDCOMPANY"); break;
		case MDF_F_FUNDPMICODE : printf ("MDF_F_FUNDPMICODE"); break;
		case MDF_F_COUNTRY : printf ("MDF_F_COUNTRY"); break;
		case MDF_F_NAV : printf ("MDF_F_NAV"); break;
		case MDF_F_CLOSENAV : printf ("MDF_F_CLOSENAV"); break;
		case MDF_F_TIS : printf ("MDF_F_TIS"); break;
		case MDF_F_CLOSETIS : printf ("MDF_F_CLOSETIS"); break;
		case MDF_F_SOURCE : printf ("MDF_F_SOURCE"); break;
		case MDF_F_S1 : printf ("MDF_F_S1"); break;
		case MDF_F_CATYPE : printf ("MDF_F_CATYPE"); break;
		case MDF_F_DIVIDEND : printf ("MDF_F_DIVIDEND"); break;
		case MDF_F_CASUBTYPE : printf ("MDF_F_CASUBTYPE"); break;
		case MDF_F_ADJUSTMENTFACTOR : printf ("MDF_F_ADJUSTMENTFACTOR"); break;
		case MDF_F_NUMBEROFSHARES : printf ("MDF_F_NUMBEROFSHARES"); break;
		case MDF_F_NUMBEROFSHARESDELTA : printf ("MDF_F_NUMBEROFSHARESDELTA"); break;
		case MDF_F_NEWSHARES : printf ("MDF_F_NEWSHARES"); break;
		case MDF_F_OLDSHARES : printf ("MDF_F_OLDSHARES"); break;
		case MDF_F_SUBSCRIPTIONPRICE : printf ("MDF_F_SUBSCRIPTIONPRICE"); break;
		case MDF_F_PERIOD : printf ("MDF_F_PERIOD"); break;
		case MDF_F_NOMINALVALUE : printf ("MDF_F_NOMINALVALUE"); break;
		case MDF_F_RECORDDATE : printf ("MDF_F_RECORDDATE"); break;
		case MDF_F_PAYMENTDATE : printf ("MDF_F_PAYMENTDATE"); break;
		case MDF_F_ANNOUNCEMENTDATE : printf ("MDF_F_ANNOUNCEMENTDATE"); break;
		case MDF_F_TID : printf ("MDF_F_TID"); break;
		case MDF_F_NEWSISLASTBLOCK : printf ("MDF_F_NEWSISLASTBLOCK"); break;
		case MDF_F_SERVERNAME : printf ("MDF_F_SERVERNAME"); break;
		case MDF_F_SERVERTIME : printf ("MDF_F_SERVERTIME"); break;
		case MDF_F_SERVERDATE : printf ("MDF_F_SERVERDATE"); break;
		case MDF_F_MIC : printf ("MDF_F_MIC"); break;
		case MDF_F_UNCHANGEDPAID : printf ("MDF_F_UNCHANGEDPAID"); break;
		case MDF_F_PLUSPAID : printf ("MDF_F_PLUSPAID"); break;
		case MDF_F_MINUSPAID : printf ("MDF_F_MINUSPAID"); break;
		case MDF_F_VWAP : printf ("MDF_F_VWAP"); break;
		case MDF_F_CLOSEVWAP : printf ("MDF_F_CLOSEVWAP"); break;
		case MDF_F_SPECIALCONDITION : printf ("MDF_F_SPECIALCONDITION"); break;
		case MDF_F_TRADESTATE : printf ("MDF_F_TRADESTATE"); break;
		case MDF_F_SALES : printf ("MDF_F_SALES"); break;
		case MDF_F_EBIT : printf ("MDF_F_EBIT"); break;
		case MDF_F_PRETAXPROFIT : printf ("MDF_F_PRETAXPROFIT"); break;
		case MDF_F_NETPROFIT : printf ("MDF_F_NETPROFIT"); break;
		case MDF_F_EPS : printf ("MDF_F_EPS"); break;
		case MDF_F_DILUTEDEPS : printf ("MDF_F_DILUTEDEPS"); break;
		case MDF_F_EBITDA : printf ("MDF_F_EBITDA"); break;
		case MDF_F_EBITA : printf ("MDF_F_EBITA"); break;
		case MDF_F_NETFININCOME : printf ("MDF_F_NETFININCOME"); break;
		case MDF_F_CLOSEPRICE1D : printf ("MDF_F_CLOSEPRICE1D"); break;
		case MDF_F_CLOSEPRICE1W : printf ("MDF_F_CLOSEPRICE1W"); break;
		case MDF_F_CLOSEPRICE1M : printf ("MDF_F_CLOSEPRICE1M"); break;
		case MDF_F_CLOSEPRICE3M : printf ("MDF_F_CLOSEPRICE3M"); break;
		case MDF_F_CLOSEPRICE6M : printf ("MDF_F_CLOSEPRICE6M"); break;
		case MDF_F_CLOSEPRICE9M : printf ("MDF_F_CLOSEPRICE9M"); break;
		case MDF_F_CLOSEPRICE1Y : printf ("MDF_F_CLOSEPRICE1Y"); break;
		case MDF_F_CLOSEPRICE2Y : printf ("MDF_F_CLOSEPRICE2Y"); break;
		case MDF_F_CLOSEPRICE5Y : printf ("MDF_F_CLOSEPRICE5Y"); break;
		case MDF_F_CLOSEPRICE10Y : printf ("MDF_F_CLOSEPRICE10Y"); break;
		case MDF_F_CLOSEPRICEWTD : printf ("MDF_F_CLOSEPRICEWTD"); break;
		case MDF_F_CLOSEPRICEMTD : printf ("MDF_F_CLOSEPRICEMTD"); break;
		case MDF_F_CLOSEPRICEQTD : printf ("MDF_F_CLOSEPRICEQTD"); break;
		case MDF_F_CLOSEPRICEYTD : printf ("MDF_F_CLOSEPRICEYTD"); break;
		case MDF_F_CLOSEPRICEPYTD : printf ("MDF_F_CLOSEPRICEPYTD"); break;
		case MDF_F_ATH : printf ("MDF_F_ATH"); break;
		case MDF_F_ATL : printf ("MDF_F_ATL"); break;
		case MDF_F_HIGHPRICE1Y : printf ("MDF_F_HIGHPRICE1Y"); break;
		case MDF_F_LOWPRICE1Y : printf ("MDF_F_LOWPRICE1Y"); break;
		case MDF_F_NEWSCODINGISIN : printf ("MDF_F_NEWSCODINGISIN"); break;
		case MDF_F_CHAIRMAN : printf ("MDF_F_CHAIRMAN"); break;
		case MDF_F_CEO : printf ("MDF_F_CEO"); break;
		case MDF_F_WEBSITE : printf ("MDF_F_WEBSITE"); break;
		case MDF_F_ORGNUM : printf ("MDF_F_ORGNUM"); break;
		case MDF_F_DESCRIPTION : printf ("MDF_F_DESCRIPTION"); break;
		case MDF_F_EQUITYRATIO : printf ("MDF_F_EQUITYRATIO"); break;
		case MDF_F_S2 : printf ("MDF_F_S2"); break;
		case MDF_F_S3 : printf ("MDF_F_S3"); break;
		case MDF_F_S4 : printf ("MDF_F_S4"); break;
		case MDF_F_S5 : printf ("MDF_F_S5"); break;
		case MDF_F_S6 : printf ("MDF_F_S6"); break;
		case MDF_F_S7 : printf ("MDF_F_S7"); break;
		case MDF_F_S8 : printf ("MDF_F_S8"); break;
		case MDF_F_S9 : printf ("MDF_F_S9"); break;
		case MDF_F_S10 : printf ("MDF_F_S10"); break;
		case MDF_F_S11 : printf ("MDF_F_S11"); break;
		case MDF_F_S12 : printf ("MDF_F_S12"); break;
		case MDF_F_S13 : printf ("MDF_F_S13"); break;
		case MDF_F_S14 : printf ("MDF_F_S14"); break;
		case MDF_F_S15 : printf ("MDF_F_S15"); break;
		case MDF_F_ATHDATE : printf ("MDF_F_ATHDATE"); break;
		case MDF_F_ATLDATE : printf ("MDF_F_ATLDATE"); break;
		case MDF_F_HIGHPRICE1YDATE : printf ("MDF_F_HIGHPRICE1YDATE"); break;
		case MDF_F_LOWPRICE1YDATE : printf ("MDF_F_LOWPRICE1YDATE"); break;
		case MDF_F_REDEMPTIONPRICE : printf ("MDF_F_REDEMPTIONPRICE"); break;
		case MDF_F_SECTOR : printf ("MDF_F_SECTOR"); break;
		case MDF_F_OPERATINGCASHFLOW : printf ("MDF_F_OPERATINGCASHFLOW"); break;
		case MDF_F_HIGHPRICEYTD : printf ("MDF_F_HIGHPRICEYTD"); break;
		case MDF_F_LOWPRICEYTD : printf ("MDF_F_LOWPRICEYTD"); break;
		case MDF_F_HIGHPRICEYTDDATE : printf ("MDF_F_HIGHPRICEYTDDATE"); break;
		case MDF_F_LOWPRICEYTDDATE : printf ("MDF_F_LOWPRICEYTDDATE"); break;
		case MDF_F_COUNT : printf ("MDF_F_COUNT"); break;
		case MDF_F_GROSSPROFIT : printf ("MDF_F_GROSSPROFIT"); break;
		case MDF_F_NETSALES : printf ("MDF_F_NETSALES"); break;
		case MDF_F_ADJUSTEDEBITA : printf ("MDF_F_ADJUSTEDEBITA"); break;
		case MDF_F_TRADEYIELD : printf ("MDF_F_TRADEYIELD"); break;
		case MDF_F_VOTINGPOWERPRC : printf ("MDF_F_VOTINGPOWERPRC"); break;
		case MDF_F_CAPITALPRC : printf ("MDF_F_CAPITALPRC"); break;
		case MDF_F_GENDERCEO : printf ("MDF_F_GENDERCEO"); break;
		case MDF_F_GENDERCHAIRMAN : printf ("MDF_F_GENDERCHAIRMAN"); break;
		case MDF_F_CEOADMISSIONDATE : printf ("MDF_F_CEOADMISSIONDATE"); break;
		case MDF_F_CHAIRMANADMISSIONDATE : printf ("MDF_F_CHAIRMANADMISSIONDATE"); break;
		case MDF_F_BIRTHYEARCEO : printf ("MDF_F_BIRTHYEARCEO"); break;
		case MDF_F_BIRTHYEARCHAIRMAN : printf ("MDF_F_BIRTHYEARCHAIRMAN"); break;
		case MDF_F_ADDRESS : printf ("MDF_F_ADDRESS"); break;
		case MDF_F_POSTALCODE : printf ("MDF_F_POSTALCODE"); break;
		case MDF_F_CITY : printf ("MDF_F_CITY"); break;
		case MDF_F_TELEPHONE : printf ("MDF_F_TELEPHONE"); break;
		case MDF_F_FAX : printf ("MDF_F_FAX"); break;
		case MDF_F_EMAIL : printf ("MDF_F_EMAIL"); break;
		case MDF_F_IMPORTANTEVENTS : printf ("MDF_F_IMPORTANTEVENTS"); break;
		case MDF_F_INTANGIBLEASSET : printf ("MDF_F_INTANGIBLEASSET"); break;
		case MDF_F_GOODWILL : printf ("MDF_F_GOODWILL"); break;
		case MDF_F_FIXEDASSET : printf ("MDF_F_FIXEDASSET"); break;
		case MDF_F_FINANCIALASSET : printf ("MDF_F_FINANCIALASSET"); break;
		case MDF_F_NONCURRENTASSET : printf ("MDF_F_NONCURRENTASSET"); break;
		case MDF_F_INVENTORY : printf ("MDF_F_INVENTORY"); break;
		case MDF_F_OTHERCURRENTASSET : printf ("MDF_F_OTHERCURRENTASSET"); break;
		case MDF_F_ACCOUNTSRECEIVABLE : printf ("MDF_F_ACCOUNTSRECEIVABLE"); break;
		case MDF_F_OTHERRECEIVABLES : printf ("MDF_F_OTHERRECEIVABLES"); break;
		case MDF_F_SHORTTERMINV : printf ("MDF_F_SHORTTERMINV"); break;
		case MDF_F_CCE : printf ("MDF_F_CCE"); break;
		case MDF_F_CURRENTASSETS : printf ("MDF_F_CURRENTASSETS"); break;
		case MDF_F_TOTALASSETS : printf ("MDF_F_TOTALASSETS"); break;
		case MDF_F_SHEQUITY : printf ("MDF_F_SHEQUITY"); break;
		case MDF_F_MINORITYINTEREST : printf ("MDF_F_MINORITYINTEREST"); break;
		case MDF_F_PROVISIONS : printf ("MDF_F_PROVISIONS"); break;
		case MDF_F_LTLIABILITIES : printf ("MDF_F_LTLIABILITIES"); break;
		case MDF_F_CURLIABILITIES : printf ("MDF_F_CURLIABILITIES"); break;
		case MDF_F_TOTSHEQLIABILITIES : printf ("MDF_F_TOTSHEQLIABILITIES"); break;
		case MDF_F_NIBL : printf ("MDF_F_NIBL"); break;
		case MDF_F_IBL : printf ("MDF_F_IBL"); break;
		case MDF_F_CASHFLOWBWC : printf ("MDF_F_CASHFLOWBWC"); break;
		case MDF_F_CASHFLOWAWC : printf ("MDF_F_CASHFLOWAWC"); break;
		case MDF_F_CASHFLOWIA : printf ("MDF_F_CASHFLOWIA"); break;
		case MDF_F_CASHFLOWFA : printf ("MDF_F_CASHFLOWFA"); break;
		case MDF_F_CASHFLOWTOTAL : printf ("MDF_F_CASHFLOWTOTAL"); break;
		case MDF_F_NUMEMPLOYEES : printf ("MDF_F_NUMEMPLOYEES"); break;
		case MDF_F_MCAP : printf ("MDF_F_MCAP"); break;
		case MDF_F_CONTRACTSIZE : printf ("MDF_F_CONTRACTSIZE"); break;
		case MDF_F_BASERATIO : printf ("MDF_F_BASERATIO"); break;
		case MDF_F_SOURCEID : printf ("MDF_F_SOURCEID"); break;
		case MDF_F_ISSUER : printf ("MDF_F_ISSUER"); break;
		case MDF_F_GENIUMID : printf ("MDF_F_GENIUMID"); break;
		case MDF_F_CLOSEPRICE3Y : printf ("MDF_F_CLOSEPRICE3Y"); break;
		case MDF_F_CLOSEPRICELD : printf ("MDF_F_CLOSEPRICELD"); break;
		case MDF_F_FUNDYEARLYMGMTFEE : printf ("MDF_F_FUNDYEARLYMGMTFEE"); break;
		case MDF_F_FUNDPPMFEE : printf ("MDF_F_FUNDPPMFEE"); break;
		case MDF_F_FUNDPPMTYPE : printf ("MDF_F_FUNDPPMTYPE"); break;
		case MDF_F_FUNDBENCHMARK : printf ("MDF_F_FUNDBENCHMARK"); break;
		case MDF_F_FUNDLEVERAGE : printf ("MDF_F_FUNDLEVERAGE"); break;
		case MDF_F_FUNDDIRECTION : printf ("MDF_F_FUNDDIRECTION"); break;
		case MDF_F_PROSPECTUS : printf ("MDF_F_PROSPECTUS"); break;
		case MDF_F_GEOFOCUSREGION : printf ("MDF_F_GEOFOCUSREGION"); break;
		case MDF_F_GEOFOCUSCOUNTRY : printf ("MDF_F_GEOFOCUSCOUNTRY"); break;
		case MDF_F_OPENINTEREST : printf ("MDF_F_OPENINTEREST"); break;
		case MDF_F_CLOSEYIELD1D : printf ("MDF_F_CLOSEYIELD1D"); break;
		case MDF_F_CLOSEYIELD1W : printf ("MDF_F_CLOSEYIELD1W"); break;
		case MDF_F_CLOSEYIELD1M : printf ("MDF_F_CLOSEYIELD1M"); break;
		case MDF_F_CLOSEYIELD3M : printf ("MDF_F_CLOSEYIELD3M"); break;
		case MDF_F_CLOSEYIELD6M : printf ("MDF_F_CLOSEYIELD6M"); break;
		case MDF_F_CLOSEYIELD9M : printf ("MDF_F_CLOSEYIELD9M"); break;
		case MDF_F_CLOSEYIELD1Y : printf ("MDF_F_CLOSEYIELD1Y"); break;
		case MDF_F_CLOSEYIELD2Y : printf ("MDF_F_CLOSEYIELD2Y"); break;
		case MDF_F_CLOSEYIELD3Y : printf ("MDF_F_CLOSEYIELD3Y"); break;
		case MDF_F_CLOSEYIELD5Y : printf ("MDF_F_CLOSEYIELD5Y"); break;
		case MDF_F_CLOSEYIELD10Y : printf ("MDF_F_CLOSEYIELD10Y"); break;
		case MDF_F_CLOSEYIELDWTD : printf ("MDF_F_CLOSEYIELDWTD"); break;
		case MDF_F_CLOSEYIELDMTD : printf ("MDF_F_CLOSEYIELDMTD"); break;
		case MDF_F_CLOSEYIELDQTD : printf ("MDF_F_CLOSEYIELDQTD"); break;
		case MDF_F_CLOSEYIELDYTD : printf ("MDF_F_CLOSEYIELDYTD"); break;
		case MDF_F_CLOSEYIELDPYTD : printf ("MDF_F_CLOSEYIELDPYTD"); break;
		case MDF_F_CLOSEYIELDLD : printf ("MDF_F_CLOSEYIELDLD"); break;
		case MDF_F_ATHYIELD : printf ("MDF_F_ATHYIELD"); break;
		case MDF_F_ATLYIELD : printf ("MDF_F_ATLYIELD"); break;
		case MDF_F_ATHYIELDDATE : printf ("MDF_F_ATHYIELDDATE"); break;
		case MDF_F_ATLYIELDDATE : printf ("MDF_F_ATLYIELDDATE"); break;
		case MDF_F_HIGHYIELD1Y : printf ("MDF_F_HIGHYIELD1Y"); break;
		case MDF_F_LOWYIELD1Y : printf ("MDF_F_LOWYIELD1Y"); break;
		case MDF_F_HIGHYIELDYTD : printf ("MDF_F_HIGHYIELDYTD"); break;
		case MDF_F_LOWYIELDYTD : printf ("MDF_F_LOWYIELDYTD"); break;
		case MDF_F_HIGHYIELDYTDDATE : printf ("MDF_F_HIGHYIELDYTDDATE"); break;
		case MDF_F_LOWYIELDYTDDATE : printf ("MDF_F_LOWYIELDYTDDATE"); break;
		case MDF_F_HIGHYIELD1YDATE : printf ("MDF_F_HIGHYIELD1YDATE"); break;
		case MDF_F_LOWYIELD1YDATE : printf ("MDF_F_LOWYIELD1YDATE"); break;
		case MDF_F_CUSIP : printf ("MDF_F_CUSIP"); break;
		case MDF_F_WKN : printf ("MDF_F_WKN"); break;
		case MDF_F_UCITS : printf ("MDF_F_UCITS"); break;
		case MDF_F_INCEPTIONDATE : printf ("MDF_F_INCEPTIONDATE"); break;
		case MDF_F_FUNDBENCHMARKINSREF : printf ("MDF_F_FUNDBENCHMARKINSREF"); break;
		case MDF_F_INSTRUMENTCLASS : printf ("MDF_F_INSTRUMENTCLASS"); break;
		case MDF_F_INSTRUMENTSUBCLASS : printf ("MDF_F_INSTRUMENTSUBCLASS"); break;
		case MDF_F_CONSTITUENTS : printf ("MDF_F_CONSTITUENTS"); break;
		case MDF_F_COUPONRATE : printf ("MDF_F_COUPONRATE"); break;
		case MDF_F_COUPONDATE : printf ("MDF_F_COUPONDATE"); break;
		case MDF_F_BARRIERPRICE : printf ("MDF_F_BARRIERPRICE"); break;
		case MDF_F_STANDARDDEVIATION3Y : printf ("MDF_F_STANDARDDEVIATION3Y"); break;
		case MDF_F_ANNUALIZEDRETURN3Y : printf ("MDF_F_ANNUALIZEDRETURN3Y"); break;
		case MDF_F_SHARPERATIO3Y : printf ("MDF_F_SHARPERATIO3Y"); break;
		case MDF_F_MORNINGSTARRATING : printf ("MDF_F_MORNINGSTARRATING"); break;
		case MDF_F_SALESFEE : printf ("MDF_F_SALESFEE"); break;
		case MDF_F_PURCHASEFEE : printf ("MDF_F_PURCHASEFEE"); break;
		case MDF_F_MINSTARTAMOUNT : printf ("MDF_F_MINSTARTAMOUNT"); break;
		case MDF_F_MINSUBSCRIPTIONAMOUNT : printf ("MDF_F_MINSUBSCRIPTIONAMOUNT"); break;
		case MDF_F_PERFORMANCEFEE : printf ("MDF_F_PERFORMANCEFEE"); break;
		case MDF_F_MINADDITIONALAMOUNT : printf ("MDF_F_MINADDITIONALAMOUNT"); break;
		case MDF_F_ANNUALIZEDRETURN5Y : printf ("MDF_F_ANNUALIZEDRETURN5Y"); break;
		case MDF_F_ANNUALIZEDRETURN10Y : printf ("MDF_F_ANNUALIZEDRETURN10Y"); break;
		case MDF_F_TRADEDTHROUGHDATE : printf ("MDF_F_TRADEDTHROUGHDATE"); break;
		case MDF_F_TOTALFEE : printf ("MDF_F_TOTALFEE"); break;
		case MDF_F_DIVIDENDTYPE : printf ("MDF_F_DIVIDENDTYPE"); break;
		case MDF_F_DIVIDENDFREQUENCY : printf ("MDF_F_DIVIDENDFREQUENCY"); break;
		case MDF_F_INSTRUMENTSUBSUBCLASS : printf ("MDF_F_INSTRUMENTSUBSUBCLASS"); break;
		case MDF_F_PRIMARYMARKETPLACE : printf ("MDF_F_PRIMARYMARKETPLACE"); break;
		case MDF_F_FISCALPERIOD : printf ("MDF_F_FISCALPERIOD"); break;
		case MDF_F_SHORTDESCRIPTION : printf ("MDF_F_SHORTDESCRIPTION"); break;
		case MDF_F_FUNDRISK : printf ("MDF_F_FUNDRISK"); break;
		case MDF_F_EUSIPA : printf ("MDF_F_EUSIPA"); break;
		case MDF_F_NEWSRANK : printf ("MDF_F_NEWSRANK"); break;
		case MDF_F_AVERAGE : printf ("MDF_F_AVERAGE"); break;
		case MDF_F_MIN : printf ("MDF_F_MIN"); break;
		case MDF_F_MAX : printf ("MDF_F_MAX"); break;
		case MDF_F_FIELDNAME : printf ("MDF_F_FIELDNAME"); break;
		case MDF_F_FIELDASPECT : printf ("MDF_F_FIELDASPECT"); break;
		case MDF_F_FIELDTYPE : printf ("MDF_F_FIELDTYPE"); break;
		case MDF_F_FUNDCOMPANY2 : printf ("MDF_F_FUNDCOMPANY2"); break;
		case MDF_F_FIELDUNIT : printf ("MDF_F_FIELDUNIT"); break;
		case MDF_F_CLOSEPRICE2W : printf ("MDF_F_CLOSEPRICE2W"); break;
		case MDF_F_CLOSEYIELD2W : printf ("MDF_F_CLOSEYIELD2W"); break;
		case MDF_F_CONVERTFROMDATE : printf ("MDF_F_CONVERTFROMDATE"); break;
		case MDF_F_CONVERTTODATE : printf ("MDF_F_CONVERTTODATE"); break;
		case MDF_F_CONVERSIONPRICE : printf ("MDF_F_CONVERSIONPRICE"); break;
		case MDF_F_DURATION : printf ("MDF_F_DURATION"); break;
		case MDF_F_SETTLEMENTTYPE : printf ("MDF_F_SETTLEMENTTYPE"); break;
		case MDF_F_VOTINGPOWER : printf ("MDF_F_VOTINGPOWER"); break;
		case MDF_F_CAP : printf ("MDF_F_CAP"); break;
		case MDF_F_IMBALANCE : printf ("MDF_F_IMBALANCE"); break;
		case MDF_F_IMBALANCEDIRECTION : printf ("MDF_F_IMBALANCEDIRECTION"); break;
		case MDF_F_CROSSTYPE : printf ("MDF_F_CROSSTYPE"); break;
		case MDF_F_TICKTABLE : printf ("MDF_F_TICKTABLE"); break;
		case MDF_F_TICKSIZES : printf ("MDF_F_TICKSIZES"); break;
		case MDF_F_PRICETYPE : printf ("MDF_F_PRICETYPE"); break;
		case MDF_F_ASIANTAILSTART : printf ("MDF_F_ASIANTAILSTART"); break;
		case MDF_F_ASIANTAILEND : printf ("MDF_F_ASIANTAILEND"); break;
		case MDF_F_LOGOTYPE : printf ("MDF_F_LOGOTYPE"); break;
		case MDF_F_ISSUERNAME : printf ("MDF_F_ISSUERNAME"); break;
		case MDF_F_CONTRACTVALUE : printf ("MDF_F_CONTRACTVALUE"); break;
		case MDF_F_CLOSEBIDPRICE1D : printf ("MDF_F_CLOSEBIDPRICE1D"); break;
		case MDF_F_CLOSEBIDYIELD1D : printf ("MDF_F_CLOSEBIDYIELD1D"); break;
		case MDF_F_CLOSEBIDPRICE1W : printf ("MDF_F_CLOSEBIDPRICE1W"); break;
		case MDF_F_CLOSEBIDYIELD1W : printf ("MDF_F_CLOSEBIDYIELD1W"); break;
		case MDF_F_FINANCIALINCOME : printf ("MDF_F_FINANCIALINCOME"); break;
		case MDF_F_FINANCIALCOST : printf ("MDF_F_FINANCIALCOST"); break;
		case MDF_F_FINANCINGLEVEL : printf ("MDF_F_FINANCINGLEVEL"); break;
		case MDF_F_PARTICIPATIONRATE : printf ("MDF_F_PARTICIPATIONRATE"); break;
		case MDF_F_ISSUEPRICE : printf ("MDF_F_ISSUEPRICE"); break;
		case MDF_F_FIINSTITUTENUMBER : printf ("MDF_F_FIINSTITUTENUMBER"); break;
		case MDF_F_DELETERECORD : printf ("MDF_F_DELETERECORD"); break;
		case MDF_F_KIID : printf ("MDF_F_KIID"); break;
		case MDF_F_CFI : printf ("MDF_F_CFI"); break;
		case MDF_F_OFFBOOKQUANTITY : printf ("MDF_F_OFFBOOKQUANTITY"); break;
		case MDF_F_OFFBOOKTURNOVER : printf ("MDF_F_OFFBOOKTURNOVER"); break;
		case MDF_F_DARKQUANTITY : printf ("MDF_F_DARKQUANTITY"); break;
		case MDF_F_DARKTURNOVER : printf ("MDF_F_DARKTURNOVER"); break;
		case MDF_F_CLOSEOFFBOOKQUANTITY : printf ("MDF_F_CLOSEOFFBOOKQUANTITY"); break;
		case MDF_F_CLOSEOFFBOOKTURNOVER : printf ("MDF_F_CLOSEOFFBOOKTURNOVER"); break;
		case MDF_F_CLOSEDARKQUANTITY : printf ("MDF_F_CLOSEDARKQUANTITY"); break;
		case MDF_F_CLOSEDARKTURNOVER : printf ("MDF_F_CLOSEDARKTURNOVER"); break;
		case MDF_F_BROKERS : printf ("MDF_F_BROKERS"); break;
		case MDF_F_INTERESTINCOME : printf ("MDF_F_INTERESTINCOME"); break;
		case MDF_F_OTHERFINANCIALINCOME : printf ("MDF_F_OTHERFINANCIALINCOME"); break;
		case MDF_F_INTERESTEXPENSE : printf ("MDF_F_INTERESTEXPENSE"); break;
		case MDF_F_OTHERFINANCIALEXPENSE : printf ("MDF_F_OTHERFINANCIALEXPENSE"); break;
		case MDF_F_MINORITYINTERESTRES : printf ("MDF_F_MINORITYINTERESTRES"); break;
		case MDF_F_ACCOUNTSPAYABLE : printf ("MDF_F_ACCOUNTSPAYABLE"); break;
		case MDF_F_EVENTLINK : printf ("MDF_F_EVENTLINK"); break;
		case MDF_F_EVENTLINKLANGUAGES : printf ("MDF_F_EVENTLINKLANGUAGES"); break;
		case MDF_F_MAXLEVEL : printf ("MDF_F_MAXLEVEL"); break;
		case MDF_F_SETTLEMENTPRICE : printf ("MDF_F_SETTLEMENTPRICE"); break;
		case MDF_F_ANNUALIZEDRETURN1Y : printf ("MDF_F_ANNUALIZEDRETURN1Y"); break;
		case MDF_F_ANNUALIZEDRETURN2Y : printf ("MDF_F_ANNUALIZEDRETURN2Y"); break;
		case MDF_F_ANNUALIZEDRETURN4Y : printf ("MDF_F_ANNUALIZEDRETURN4Y"); break;
		case MDF_F_N1 : printf ("MDF_F_N1"); break;
		case MDF_F_N2 : printf ("MDF_F_N2"); break;
		case MDF_F_N3 : printf ("MDF_F_N3"); break;
		case MDF_F_N4 : printf ("MDF_F_N4"); break;
		case MDF_F_N5 : printf ("MDF_F_N5"); break;
		case MDF_F_N6 : printf ("MDF_F_N6"); break;
		case MDF_F_N7 : printf ("MDF_F_N7"); break;
		case MDF_F_N8 : printf ("MDF_F_N8"); break;
		case MDF_F_N9 : printf ("MDF_F_N9"); break;
		case MDF_F_N10 : printf ("MDF_F_N10"); break;
		case MDF_F_I1 : printf ("MDF_F_I1"); break;
		case MDF_F_I2 : printf ("MDF_F_I2"); break;
		case MDF_F_I3 : printf ("MDF_F_I3"); break;
		case MDF_F_I4 : printf ("MDF_F_I4"); break;
		case MDF_F_I5 : printf ("MDF_F_I5"); break;
		case MDF_F_I6 : printf ("MDF_F_I6"); break;
		case MDF_F_I7 : printf ("MDF_F_I7"); break;
		case MDF_F_I8 : printf ("MDF_F_I8"); break;
		case MDF_F_I9 : printf ("MDF_F_I9"); break;
		case MDF_F_I10 : printf ("MDF_F_I10"); break;
		case MDF_F_INSREF1 : printf ("MDF_F_INSREF1"); break;
		case MDF_F_INSREF2 : printf ("MDF_F_INSREF2"); break;
		case MDF_F_INSREF3 : printf ("MDF_F_INSREF3"); break;
		case MDF_F_INSREF4 : printf ("MDF_F_INSREF4"); break;
		case MDF_F_INSREF5 : printf ("MDF_F_INSREF5"); break;
		case MDF_F_D1 : printf ("MDF_F_D1"); break;
		case MDF_F_D2 : printf ("MDF_F_D2"); break;
		case MDF_F_D3 : printf ("MDF_F_D3"); break;
		case MDF_F_CITYPE : printf ("MDF_F_CITYPE"); break;
		case MDF_F_CISUBTYPE : printf ("MDF_F_CISUBTYPE"); break;
		case MDF_F_SEQUENCE : printf ("MDF_F_SEQUENCE"); break;
		case MDF_F_OUTSTANDINGAMOUNT : printf ("MDF_F_OUTSTANDINGAMOUNT"); break;
		case MDF_F_INTERESTRATE : printf ("MDF_F_INTERESTRATE"); break;
		case MDF_F_MARKETMAKER : printf ("MDF_F_MARKETMAKER"); break;
		case MDF_F_SUSTAINDESCRIPTION : printf ("MDF_F_SUSTAINDESCRIPTION"); break;
		case MDF_F_SUSTAININVESTING1 : printf ("MDF_F_SUSTAININVESTING1"); break;
		case MDF_F_SUSTAININVESTING2 : printf ("MDF_F_SUSTAININVESTING2"); break;
		case MDF_F_SUSTAINDIVESTING1 : printf ("MDF_F_SUSTAINDIVESTING1"); break;
		case MDF_F_SUSTAINDIVESTING2 : printf ("MDF_F_SUSTAINDIVESTING2"); break;
		case MDF_F_SUSTAININFLUENCE : printf ("MDF_F_SUSTAININFLUENCE"); break;
		case MDF_F_SUSTAINMONITORING1 : printf ("MDF_F_SUSTAINMONITORING1"); break;
		case MDF_F_SUSTAINMONITORING2 : printf ("MDF_F_SUSTAINMONITORING2"); break;
		case MDF_F_SUSTAINCOMMENT : printf ("MDF_F_SUSTAINCOMMENT"); break;
		case MDF_F_NUMBEROFPREFSHARES : printf ("MDF_F_NUMBEROFPREFSHARES"); break;
		case MDF_F_MARKETOPEN : printf ("MDF_F_MARKETOPEN"); break;
		case MDF_F_MARKETCLOSE : printf ("MDF_F_MARKETCLOSE"); break;
		case MDF_F_MARKETEARLYCLOSE : printf ("MDF_F_MARKETEARLYCLOSE"); break;
		case MDF_F_LEGALSTRUCTURE : printf ("MDF_F_LEGALSTRUCTURE"); break;
		case MDF_F_ONGOINGCHARGE : printf ("MDF_F_ONGOINGCHARGE"); break;
		case MDF_F_PRICINGFREQUENCY : printf ("MDF_F_PRICINGFREQUENCY"); break;
		case MDF_F_MARKETOPENDAYS : printf ("MDF_F_MARKETOPENDAYS"); break;
		case MDF_F_CLOSETRADEPRICE : printf ("MDF_F_CLOSETRADEPRICE"); break;
		case MDF_F_CLOSEPRICETYPE : printf ("MDF_F_CLOSEPRICETYPE"); break;
		case MDF_F_NETDIVIDEND : printf ("MDF_F_NETDIVIDEND"); break;
		case MDF_F_PRODUCTCODE : printf ("MDF_F_PRODUCTCODE"); break;
		case MDF_F_QUOTINGTYPE : printf ("MDF_F_QUOTINGTYPE"); break;
		case MDF_F_TRADEAGREEMENTTIME : printf ("MDF_F_TRADEAGREEMENTTIME"); break;
		case MDF_F_TRADEAGREEMENTDATE : printf ("MDF_F_TRADEAGREEMENTDATE"); break;
		case MDF_F_LATESTYEARENDREPORT : printf ("MDF_F_LATESTYEARENDREPORT"); break;
		case MDF_F_VOLUMEDIMENSION : printf ("MDF_F_VOLUMEDIMENSION"); break;
		case MDF_F_NEWSCODINGREGULATORY : printf ("MDF_F_NEWSCODINGREGULATORY"); break;
		case MDF_F_NORMANAMOUNT : printf ("MDF_F_NORMANAMOUNT"); break;
		case MDF_F_CSR : printf ("MDF_F_CSR"); break;
		case MDF_F_CIK : printf ("MDF_F_CIK"); break;
		case MDF_F_PRIIP00010 : printf ("MDF_F_PRIIP00010"); break;
		case MDF_F_PRIIP00020 : printf ("MDF_F_PRIIP00020"); break;
		case MDF_F_PRIIP00030 : printf ("MDF_F_PRIIP00030"); break;
		case MDF_F_PRIIP00040 : printf ("MDF_F_PRIIP00040"); break;
		case MDF_F_PRIIP00050 : printf ("MDF_F_PRIIP00050"); break;
		case MDF_F_PRIIP00060 : printf ("MDF_F_PRIIP00060"); break;
		case MDF_F_PRIIPKID : printf ("MDF_F_PRIIPKID"); break;
		case MDF_F_PRIIP00080 : printf ("MDF_F_PRIIP00080"); break;
		case MDF_F_PRIIP00090 : printf ("MDF_F_PRIIP00090"); break;
		case MDF_F_PRIIP00100 : printf ("MDF_F_PRIIP00100"); break;
		case MDF_F_PRIIP01010 : printf ("MDF_F_PRIIP01010"); break;
		case MDF_F_PRIIP01020 : printf ("MDF_F_PRIIP01020"); break;
		case MDF_F_PRIIP01030 : printf ("MDF_F_PRIIP01030"); break;
		case MDF_F_PRIIP01040 : printf ("MDF_F_PRIIP01040"); break;
		case MDF_F_PRIIP01050 : printf ("MDF_F_PRIIP01050"); break;
		case MDF_F_PRIIP01060 : printf ("MDF_F_PRIIP01060"); break;
		case MDF_F_PRIIP01070 : printf ("MDF_F_PRIIP01070"); break;
		case MDF_F_PRIIP01080 : printf ("MDF_F_PRIIP01080"); break;
		case MDF_F_PRIIP01090 : printf ("MDF_F_PRIIP01090"); break;
		case MDF_F_PRIIP01100 : printf ("MDF_F_PRIIP01100"); break;
		case MDF_F_PRIIP01110 : printf ("MDF_F_PRIIP01110"); break;
		case MDF_F_PRIIP01120 : printf ("MDF_F_PRIIP01120"); break;
		case MDF_F_PRIIP01140 : printf ("MDF_F_PRIIP01140"); break;
		case MDF_F_PRIIP02010 : printf ("MDF_F_PRIIP02010"); break;
		case MDF_F_PRIIP02020 : printf ("MDF_F_PRIIP02020"); break;
		case MDF_F_PRIIP02030 : printf ("MDF_F_PRIIP02030"); break;
		case MDF_F_PRIIP02040 : printf ("MDF_F_PRIIP02040"); break;
		case MDF_F_PRIIP02050 : printf ("MDF_F_PRIIP02050"); break;
		case MDF_F_PRIIP02060 : printf ("MDF_F_PRIIP02060"); break;
		case MDF_F_PRIIP02070 : printf ("MDF_F_PRIIP02070"); break;
		case MDF_F_PRIIP02080 : printf ("MDF_F_PRIIP02080"); break;
		case MDF_F_PRIIP02090 : printf ("MDF_F_PRIIP02090"); break;
		case MDF_F_PRIIP02100 : printf ("MDF_F_PRIIP02100"); break;
		case MDF_F_PRIIP02110 : printf ("MDF_F_PRIIP02110"); break;
		case MDF_F_PRIIP02120 : printf ("MDF_F_PRIIP02120"); break;
		case MDF_F_PRIIP02130 : printf ("MDF_F_PRIIP02130"); break;
		case MDF_F_PRIIP02140 : printf ("MDF_F_PRIIP02140"); break;
		case MDF_F_PRIIP02150 : printf ("MDF_F_PRIIP02150"); break;
		case MDF_F_PRIIP02160 : printf ("MDF_F_PRIIP02160"); break;
		case MDF_F_PRIIP02170 : printf ("MDF_F_PRIIP02170"); break;
		case MDF_F_PRIIP02180 : printf ("MDF_F_PRIIP02180"); break;
		case MDF_F_PRIIP03010 : printf ("MDF_F_PRIIP03010"); break;
		case MDF_F_PRIIP03015 : printf ("MDF_F_PRIIP03015"); break;
		case MDF_F_PRIIP03020 : printf ("MDF_F_PRIIP03020"); break;
		case MDF_F_PRIIP03030 : printf ("MDF_F_PRIIP03030"); break;
		case MDF_F_PRIIP03040 : printf ("MDF_F_PRIIP03040"); break;
		case MDF_F_PRIIP03050 : printf ("MDF_F_PRIIP03050"); break;
		case MDF_F_PRIIP03060 : printf ("MDF_F_PRIIP03060"); break;
		case MDF_F_PRIIP03070 : printf ("MDF_F_PRIIP03070"); break;
		case MDF_F_PRIIP03080 : printf ("MDF_F_PRIIP03080"); break;
		case MDF_F_PRIIP03090 : printf ("MDF_F_PRIIP03090"); break;
		case MDF_F_PRIIP03095 : printf ("MDF_F_PRIIP03095"); break;
		case MDF_F_PRIIP03100 : printf ("MDF_F_PRIIP03100"); break;
		case MDF_F_PRIIP03105 : printf ("MDF_F_PRIIP03105"); break;
		case MDF_F_SUBMARKET : printf ("MDF_F_SUBMARKET"); break;
		case MDF_F_PRIIP04020 : printf ("MDF_F_PRIIP04020"); break;
		case MDF_F_PRIIP04030 : printf ("MDF_F_PRIIP04030"); break;
		case MDF_F_PRIIP04040 : printf ("MDF_F_PRIIP04040"); break;
		case MDF_F_PRIIP04050 : printf ("MDF_F_PRIIP04050"); break;
		case MDF_F_PRIIP04060 : printf ("MDF_F_PRIIP04060"); break;
		case MDF_F_PRIIP04070 : printf ("MDF_F_PRIIP04070"); break;
		case MDF_F_PRIIP04080 : printf ("MDF_F_PRIIP04080"); break;
		case MDF_F_PRIIP04081 : printf ("MDF_F_PRIIP04081"); break;
		case MDF_F_PRIIP04082 : printf ("MDF_F_PRIIP04082"); break;
		case MDF_F_PRIIP04083 : printf ("MDF_F_PRIIP04083"); break;
		case MDF_F_PRIIP04084 : printf ("MDF_F_PRIIP04084"); break;
		case MDF_F_PRIIP04085 : printf ("MDF_F_PRIIP04085"); break;
		case MDF_F_PRIIP04090 : printf ("MDF_F_PRIIP04090"); break;
		case MDF_F_PRIIP04100 : printf ("MDF_F_PRIIP04100"); break;
		case MDF_F_PRIIP04110 : printf ("MDF_F_PRIIP04110"); break;
		case MDF_F_PRIIP05010 : printf ("MDF_F_PRIIP05010"); break;
		case MDF_F_PRIIP05020 : printf ("MDF_F_PRIIP05020"); break;
		case MDF_F_PRIIP05030 : printf ("MDF_F_PRIIP05030"); break;
		case MDF_F_PRIIP05040 : printf ("MDF_F_PRIIP05040"); break;
		case MDF_F_PRIIP05050 : printf ("MDF_F_PRIIP05050"); break;
		case MDF_F_PRIIP05060 : printf ("MDF_F_PRIIP05060"); break;
		case MDF_F_PRIIP05065 : printf ("MDF_F_PRIIP05065"); break;
		case MDF_F_PRIIP05070 : printf ("MDF_F_PRIIP05070"); break;
		case MDF_F_PRIIP05080 : printf ("MDF_F_PRIIP05080"); break;
		case MDF_F_MMT : printf ("MDF_F_MMT"); break;
		case MDF_F_FISN : printf ("MDF_F_FISN"); break;
		case MDF_F_LEI : printf ("MDF_F_LEI"); break;
		case MDF_F_SECTORMEMBERS : printf ("MDF_F_SECTORMEMBERS"); break;
		case MDF_F_PRIIP06010 : printf ("MDF_F_PRIIP06010"); break;
		case MDF_F_PRIIP06020 : printf ("MDF_F_PRIIP06020"); break;
		case MDF_F_PRIIP06030 : printf ("MDF_F_PRIIP06030"); break;
		case MDF_F_PRIIP06040 : printf ("MDF_F_PRIIP06040"); break;
		case MDF_F_PRIIP06050 : printf ("MDF_F_PRIIP06050"); break;
		case MDF_F_PRIIP06060 : printf ("MDF_F_PRIIP06060"); break;
		case MDF_F_PRIIP06070 : printf ("MDF_F_PRIIP06070"); break;
		case MDF_F_PRIIP06080 : printf ("MDF_F_PRIIP06080"); break;
		case MDF_F_PRIIP07010 : printf ("MDF_F_PRIIP07010"); break;
		case MDF_F_PRIIP07020 : printf ("MDF_F_PRIIP07020"); break;
		case MDF_F_PRIIP07030 : printf ("MDF_F_PRIIP07030"); break;
		case MDF_F_PRIIP07040 : printf ("MDF_F_PRIIP07040"); break;
		case MDF_F_PRIIP07050 : printf ("MDF_F_PRIIP07050"); break;
		case MDF_F_PRIIP07060 : printf ("MDF_F_PRIIP07060"); break;
		case MDF_F_PRIIP07070 : printf ("MDF_F_PRIIP07070"); break;
		case MDF_F_PRIIP07080 : printf ("MDF_F_PRIIP07080"); break;
		case MDF_F_PRIIP07090 : printf ("MDF_F_PRIIP07090"); break;
		case MDF_F_PRIIP07100 : printf ("MDF_F_PRIIP07100"); break;
		case MDF_F_PRIIP07110 : printf ("MDF_F_PRIIP07110"); break;
		case MDF_F_PRIIP07120 : printf ("MDF_F_PRIIP07120"); break;
		case MDF_F_MIFID00010 : printf ("MDF_F_MIFID00010"); break;
		case MDF_F_MIFID00020 : printf ("MDF_F_MIFID00020"); break;
		case MDF_F_MIFID00030 : printf ("MDF_F_MIFID00030"); break;
		case MDF_F_MIFID00040 : printf ("MDF_F_MIFID00040"); break;
		case MDF_F_NEWSIDREPLACE : printf ("MDF_F_NEWSIDREPLACE"); break;
		case MDF_F_MIFID00060 : printf ("MDF_F_MIFID00060"); break;
		case MDF_F_MIFID00070 : printf ("MDF_F_MIFID00070"); break;
		case MDF_F_MIFID00080 : printf ("MDF_F_MIFID00080"); break;
		case MDF_F_MIFID00090 : printf ("MDF_F_MIFID00090"); break;
		case MDF_F_MIFID00100 : printf ("MDF_F_MIFID00100"); break;
		case MDF_F_MIFID01010 : printf ("MDF_F_MIFID01010"); break;
		case MDF_F_MIFID01020 : printf ("MDF_F_MIFID01020"); break;
		case MDF_F_MIFID01030 : printf ("MDF_F_MIFID01030"); break;
		case MDF_F_MIFID02010 : printf ("MDF_F_MIFID02010"); break;
		case MDF_F_MIFID02020 : printf ("MDF_F_MIFID02020"); break;
		case MDF_F_MIFID02030 : printf ("MDF_F_MIFID02030"); break;
		case MDF_F_MIFID02040 : printf ("MDF_F_MIFID02040"); break;
		case MDF_F_MIFID03010 : printf ("MDF_F_MIFID03010"); break;
		case MDF_F_MIFID03020 : printf ("MDF_F_MIFID03020"); break;
		case MDF_F_MIFID03030 : printf ("MDF_F_MIFID03030"); break;
		case MDF_F_MIFID03040 : printf ("MDF_F_MIFID03040"); break;
		case MDF_F_MIFID03050 : printf ("MDF_F_MIFID03050"); break;
		case MDF_F_MIFID04010 : printf ("MDF_F_MIFID04010"); break;
		case MDF_F_MIFID04020 : printf ("MDF_F_MIFID04020"); break;
		case MDF_F_MIFID04030 : printf ("MDF_F_MIFID04030"); break;
		case MDF_F_MIFID04040 : printf ("MDF_F_MIFID04040"); break;
		case MDF_F_MIFID04050 : printf ("MDF_F_MIFID04050"); break;
		case MDF_F_MIFID05010 : printf ("MDF_F_MIFID05010"); break;
		case MDF_F_MIFID05020 : printf ("MDF_F_MIFID05020"); break;
		case MDF_F_MIFID05030 : printf ("MDF_F_MIFID05030"); break;
		case MDF_F_MIFID05040 : printf ("MDF_F_MIFID05040"); break;
		case MDF_F_MIFID05050 : printf ("MDF_F_MIFID05050"); break;
		case MDF_F_MIFID05060 : printf ("MDF_F_MIFID05060"); break;
		case MDF_F_MIFID05070 : printf ("MDF_F_MIFID05070"); break;
		case MDF_F_MIFID05080 : printf ("MDF_F_MIFID05080"); break;
		case MDF_F_MIFID05080N : printf ("MDF_F_MIFID05080N"); break;
		case MDF_F_ENTITLEMENTPACKAGE : printf ("MDF_F_ENTITLEMENTPACKAGE"); break;
		case MDF_F_MIFID05100 : printf ("MDF_F_MIFID05100"); break;
		case MDF_F_MIFID05110 : printf ("MDF_F_MIFID05110"); break;
		case MDF_F_MIFID06010 : printf ("MDF_F_MIFID06010"); break;
		case MDF_F_MIFID06020 : printf ("MDF_F_MIFID06020"); break;
		case MDF_F_MIFID06030 : printf ("MDF_F_MIFID06030"); break;
		case MDF_F_MIFID06040 : printf ("MDF_F_MIFID06040"); break;
		case MDF_F_MIFID07010 : printf ("MDF_F_MIFID07010"); break;
		case MDF_F_MIFID07020 : printf ("MDF_F_MIFID07020"); break;
		case MDF_F_MIFID07030 : printf ("MDF_F_MIFID07030"); break;
		case MDF_F_MIFID07040 : printf ("MDF_F_MIFID07040"); break;
		case MDF_F_MIFID07050 : printf ("MDF_F_MIFID07050"); break;
		case MDF_F_MIFID07060 : printf ("MDF_F_MIFID07060"); break;
		case MDF_F_MIFID07070 : printf ("MDF_F_MIFID07070"); break;
		case MDF_F_MIFID07080 : printf ("MDF_F_MIFID07080"); break;
		case MDF_F_MIFID07090 : printf ("MDF_F_MIFID07090"); break;
		case MDF_F_MIFID07100 : printf ("MDF_F_MIFID07100"); break;
		case MDF_F_MIFID07110 : printf ("MDF_F_MIFID07110"); break;
		case MDF_F_MIFID07120 : printf ("MDF_F_MIFID07120"); break;
		case MDF_F_MIFID07130 : printf ("MDF_F_MIFID07130"); break;
		case MDF_F_MIFID07140 : printf ("MDF_F_MIFID07140"); break;
		case MDF_F_MIFID08010 : printf ("MDF_F_MIFID08010"); break;
		case MDF_F_MIFID08020 : printf ("MDF_F_MIFID08020"); break;
		case MDF_F_MIFID08030 : printf ("MDF_F_MIFID08030"); break;
		case MDF_F_MIFID08040 : printf ("MDF_F_MIFID08040"); break;
		case MDF_F_MIFID08050 : printf ("MDF_F_MIFID08050"); break;
		case MDF_F_MIFID08060 : printf ("MDF_F_MIFID08060"); break;
		case MDF_F_MIFID08070 : printf ("MDF_F_MIFID08070"); break;
		case MDF_F_MIFID08080 : printf ("MDF_F_MIFID08080"); break;
		case MDF_F_MIFID08090 : printf ("MDF_F_MIFID08090"); break;
		case MDF_F_MIFID08100 : printf ("MDF_F_MIFID08100"); break;
		case MDF_F_OPERATINGMIC : printf ("MDF_F_OPERATINGMIC"); break;
		case MDF_F_CVR : printf ("MDF_F_CVR"); break;
		case MDF_F_KENNITALA : printf ("MDF_F_KENNITALA"); break;
		case MDF_F_YTUNNUS : printf ("MDF_F_YTUNNUS"); break;
		case MDF_F_ORGNUMNO : printf ("MDF_F_ORGNUMNO"); break;
		case MDF_F_NEWSCODINGTAGS : printf ("MDF_F_NEWSCODINGTAGS"); break;
		case MDF_F_NEWSCODINGIMPACT : printf ("MDF_F_NEWSCODINGIMPACT"); break;
		case MDF_F_ACTIVESHARE : printf ("MDF_F_ACTIVESHARE"); break;
		case MDF_F_TRACKINGERROR : printf ("MDF_F_TRACKINGERROR"); break;
		case MDF_F_HIGHPRICE3Y : printf ("MDF_F_HIGHPRICE3Y"); break;
		case MDF_F_LOWPRICE3Y : printf ("MDF_F_LOWPRICE3Y"); break;
		case MDF_F_HIGHPRICE5Y : printf ("MDF_F_HIGHPRICE5Y"); break;
		case MDF_F_LOWPRICE5Y : printf ("MDF_F_LOWPRICE5Y"); break;
		case MDF_F_HIGHPRICE10Y : printf ("MDF_F_HIGHPRICE10Y"); break;
		case MDF_F_LOWPRICE10Y : printf ("MDF_F_LOWPRICE10Y"); break;
		case MDF_F_DPS : printf ("MDF_F_DPS"); break;
		case MDF_F_CFPS : printf ("MDF_F_CFPS"); break;
		case MDF_F_CFPSTTM : printf ("MDF_F_CFPSTTM"); break;
		case MDF_F_CFPSLAST : printf ("MDF_F_CFPSLAST"); break;
		case MDF_F_EPSTTM : printf ("MDF_F_EPSTTM"); break;
		case MDF_F_EPSLAST : printf ("MDF_F_EPSLAST"); break;
		case MDF_F_SPS : printf ("MDF_F_SPS"); break;
		case MDF_F_SPSTTM : printf ("MDF_F_SPSTTM"); break;
		case MDF_F_SPSLAST : printf ("MDF_F_SPSLAST"); break;
		case MDF_F_BVPS : printf ("MDF_F_BVPS"); break;
		case MDF_F_BVPSLAST : printf ("MDF_F_BVPSLAST"); break;
		case MDF_F_ROE : printf ("MDF_F_ROE"); break;
		case MDF_F_ROETTM : printf ("MDF_F_ROETTM"); break;
		case MDF_F_ROELAST : printf ("MDF_F_ROELAST"); break;
		case MDF_F_ROA : printf ("MDF_F_ROA"); break;
		case MDF_F_ROATTM : printf ("MDF_F_ROATTM"); break;
		case MDF_F_ROALAST : printf ("MDF_F_ROALAST"); break;
		case MDF_F_GM : printf ("MDF_F_GM"); break;
		case MDF_F_GMTTM : printf ("MDF_F_GMTTM"); break;
		case MDF_F_GMLAST : printf ("MDF_F_GMLAST"); break;
		case MDF_F_OM : printf ("MDF_F_OM"); break;
		case MDF_F_OMTTM : printf ("MDF_F_OMTTM"); break;
		case MDF_F_OMLAST : printf ("MDF_F_OMLAST"); break;
		case MDF_F_PM : printf ("MDF_F_PM"); break;
		case MDF_F_PMTTM : printf ("MDF_F_PMTTM"); break;
		case MDF_F_PMLAST : printf ("MDF_F_PMLAST"); break;
		case MDF_F_MIFID00001 : printf ("MDF_F_MIFID00001"); break;
		case MDF_F_MIFID00005 : printf ("MDF_F_MIFID00005"); break;
		case MDF_F_MIFID00006 : printf ("MDF_F_MIFID00006"); break;
		case MDF_F_MIFID00007 : printf ("MDF_F_MIFID00007"); break;
		case MDF_F_MIFID00008 : printf ("MDF_F_MIFID00008"); break;
		case MDF_F_MIFID00045 : printf ("MDF_F_MIFID00045"); break;
		case MDF_F_MIFID00047 : printf ("MDF_F_MIFID00047"); break;
		case MDF_F_MIFID00002 : printf ("MDF_F_MIFID00002"); break;
		case MDF_F_MIFID00003 : printf ("MDF_F_MIFID00003"); break;
		case MDF_F_MIFID00073 : printf ("MDF_F_MIFID00073"); break;
		case MDF_F_MIFID00074 : printf ("MDF_F_MIFID00074"); break;
		case MDF_F_MIFID00075 : printf ("MDF_F_MIFID00075"); break;
		case MDF_F_MIFID00085 : printf ("MDF_F_MIFID00085"); break;
		case MDF_F_MIFID00095 : printf ("MDF_F_MIFID00095"); break;
		case MDF_F_MIFID00110 : printf ("MDF_F_MIFID00110"); break;
		case MDF_F_MIFID00120 : printf ("MDF_F_MIFID00120"); break;
		case MDF_F_MIFID00004 : printf ("MDF_F_MIFID00004"); break;
		case MDF_F_MIFID01000 : printf ("MDF_F_MIFID01000"); break;
		case MDF_F_MIFID05105 : printf ("MDF_F_MIFID05105"); break;
		case MDF_F_MIFID05115 : printf ("MDF_F_MIFID05115"); break;
		case MDF_F_MIFID07025 : printf ("MDF_F_MIFID07025"); break;
		case MDF_F_MIFID07105 : printf ("MDF_F_MIFID07105"); break;
		case MDF_F_MIFID07150 : printf ("MDF_F_MIFID07150"); break;
		case MDF_F_MIFID07155 : printf ("MDF_F_MIFID07155"); break;
		case MDF_F_MIFID07160 : printf ("MDF_F_MIFID07160"); break;
		case MDF_F_MIFID08015 : printf ("MDF_F_MIFID08015"); break;
		case MDF_F_MIFID08025 : printf ("MDF_F_MIFID08025"); break;
		case MDF_F_MIFID08045 : printf ("MDF_F_MIFID08045"); break;
		case MDF_F_MIFID08110 : printf ("MDF_F_MIFID08110"); break;
		case MDF_F_MIFID08120 : printf ("MDF_F_MIFID08120"); break;
		case MDF_F_FIGI : printf ("MDF_F_FIGI"); break;
		case MDF_F_FIGICOMPOSITE : printf ("MDF_F_FIGICOMPOSITE"); break;
		case MDF_F_FIGISHARECLASS : printf ("MDF_F_FIGISHARECLASS"); break;
		case MDF_F_MMO : printf ("MDF_F_MMO"); break;
		case MDF_F_FIGISECURITYTYPE : printf ("MDF_F_FIGISECURITYTYPE"); break;
		case MDF_F_COUPONFREQUENCY : printf ("MDF_F_COUPONFREQUENCY"); break;
		case MDF_F_ORDERID : printf ("MDF_F_ORDERID"); break;
		case MDF_F_ORDERIDSOURCE : printf ("MDF_F_ORDERIDSOURCE"); break;
		case MDF_F_ORDERPRIORITY : printf ("MDF_F_ORDERPRIORITY"); break;
		case MDF_F_IV : printf ("MDF_F_IV"); break;
		case MDF_F_DELTA : printf ("MDF_F_DELTA"); break;
		case MDF_F_GAMMA : printf ("MDF_F_GAMMA"); break;
		case MDF_F_RHO : printf ("MDF_F_RHO"); break;
		case MDF_F_THETA : printf ("MDF_F_THETA"); break;
		case MDF_F_VEGA : printf ("MDF_F_VEGA"); break;
		case MDF_F_ORDERSIDE : printf ("MDF_F_ORDERSIDE"); break;
		case MDF_F_ORDERPARTICIPANT : printf ("MDF_F_ORDERPARTICIPANT"); break;
		case MDF_F_ORDERPRICE : printf ("MDF_F_ORDERPRICE"); break;
		case MDF_F_ORDERQUANTITY : printf ("MDF_F_ORDERQUANTITY"); break;
		case MDF_F_IVBID : printf ("MDF_F_IVBID"); break;
		case MDF_F_IVASK : printf ("MDF_F_IVASK"); break;
		case MDF_F_DAYCOUNTCONVENTION : printf ("MDF_F_DAYCOUNTCONVENTION"); break;
		case MDF_F_EXPIRATIONTYPE : printf ("MDF_F_EXPIRATIONTYPE"); break;
		case MDF_F_TENOR : printf ("MDF_F_TENOR"); break;
		case MDF_F_COMBOLEGS : printf ("MDF_F_COMBOLEGS"); break;
		case MDF_F_SWINGFACTOR : printf ("MDF_F_SWINGFACTOR"); break;
		case MDF_F_SWINGFACTORSUB : printf ("MDF_F_SWINGFACTORSUB"); break;
		case MDF_F_SWINGMETHOD : printf ("MDF_F_SWINGMETHOD"); break;
		case MDF_F_INCOMEPROPMAN : printf ("MDF_F_INCOMEPROPMAN"); break;
		case MDF_F_CLEARING : printf ("MDF_F_CLEARING"); break;
		case MDF_F_LASTMOD : printf ("MDF_F_LASTMOD"); break;
		case MDF_F_APPLICABLENAV : printf ("MDF_F_APPLICABLENAV"); break;
		case MDF_F_FUNDCUTOFFTIME : printf ("MDF_F_FUNDCUTOFFTIME"); break;
		case MDF_F_EVENTSTATUS : printf ("MDF_F_EVENTSTATUS"); break;
		case MDF_F_AVGTURNOVER1W : printf ("MDF_F_AVGTURNOVER1W"); break;
		case MDF_F_AVGTURNOVER1M : printf ("MDF_F_AVGTURNOVER1M"); break;
		case MDF_F_AVGTURNOVER3M : printf ("MDF_F_AVGTURNOVER3M"); break;
		case MDF_F_AVGTURNOVER1Y : printf ("MDF_F_AVGTURNOVER1Y"); break;
		case MDF_F_AVGTURNOVERYTD : printf ("MDF_F_AVGTURNOVERYTD"); break;
		case MDF_F_SMA20 : printf ("MDF_F_SMA20"); break;
		case MDF_F_SMA50 : printf ("MDF_F_SMA50"); break;
		case MDF_F_SMA200 : printf ("MDF_F_SMA200"); break;
		case MDF_F_RSI14 : printf ("MDF_F_RSI14"); break;
		case MDF_F_MACD : printf ("MDF_F_MACD"); break;
		case MDF_F_TRADESTATEACTIONS : printf ("MDF_F_TRADESTATEACTIONS"); break;
		case MDF_F_SHARECLASS : printf ("MDF_F_SHARECLASS"); break;

		default : printf ("unknown");
	}

	printf (" = %s\n", value);

	if (   tag == MDF_F_CONSTITUENTS || tag == MDF_F_TICKSIZES      || tag == MDF_F_BROKERS
	    || tag == MDF_F_UNDERLYINGID || tag == MDF_F_NEWSCODINGTAGS || tag == MDF_F_COMBOLEGS) {
		printf ("\tThis data is in tabular format, let's break up the data in rows and columns:\n");
		print_tabular (value);
	}
}

static char *get_next_row (char *value, char ** const row)
{
	int is_escaped = 0;

	if (value == NULL || *value == '\0' || row == NULL)
		return NULL;

	for (*row = value; *value != '\0'; value++) {
		if (is_escaped == 1) {
			is_escaped = 0;
			continue;
		}

		if (*value == '|') {
			*value++ = '\0';
			return value;
		}

		if (*value == '\\')
			is_escaped = 1;
	}

	return value;
}

static char *get_next_column (char *row, char ** const column)
{
	char *dst;
	int is_escaped = 0;

	if (row == NULL || *row == '\0' || column == NULL)
		return NULL;

	for (*column = row, dst = row; *row != '\0'; row++) {
		if (is_escaped == 1) {
			is_escaped = 0;
			*dst++ = *row;
			continue;
		}

		if (*row == ' ') {
			*dst = '\0';
			return row + 1;
		}

		if (*row == '\\')
			is_escaped = 1;
		else
			*dst++ = *row;
	}

	*dst = '\0';
	return row;
}

/* this function will take a tabular field and split it into it's rows and columns.
 * The columns will be printed with a tab separation and a newline will be added 
 * between rows */
void print_tabular (char *value)
{
	char *row;
	int num = 0;

	while ((value = get_next_row (value, &row))) {
		char *column;

		printf ("\tRow #%d: ", num++);

		while ((row = get_next_column (row, &column)))
			printf ("%s\t", column);

		printf ("\n");
	}
}

/* This function will extract the underlying name from the name of a derivative.
 * The resulting string needs to be freed by the caller. */
char *get_underlying_name (const char * const string)
{
	char *p, *name;

	if (string == NULL)
		return NULL;

	p = strstr (string, " (");

	if (p == NULL)
		return strdup (string);

	*p = '\0';
	name = strdup (string);
	*p = ' ';

	return name;
}

